<?php

use Zen\Payment\Api;
use Zen\Payment\CartData;
use Zen\Payment\Paywall;
use Zen\Payment\Util;

include_once(Mage::getConfig()->getModuleDir('helper', 'Zen') . '/Helper/payment-core/src/Api.php');
include_once(Mage::getConfig()->getModuleDir('helper', 'Zen') . '/Helper/payment-core/src/Util.php');
include_once(Mage::getConfig()->getModuleDir('helper', 'Zen') . '/Helper/payment-core/src/Paywall.php');
include_once(Mage::getConfig()->getModuleDir('helper', 'Zen') . '/Helper/payment-core/src/CartData.php');

/**
 * Class Zen_Model_PaymentMethod
 */
class Zen_Model_PaymentMethod extends Mage_Payment_Model_Method_Abstract
{

	protected $_code = 'zen';
	protected $_formBlockType = 'zen/form';
	protected $_canUseCheckout = true;
	protected $_canVoid = true;
	protected $_isInitializeNeeded = true;
	protected $_isGateway = true;
	protected $_canOrder = true;
	protected $_canReviewPayment = true;
	protected $_canRefund = true;

	/**
	 * Refund payment
	 *
	 * @param Varien_Object $payment
	 * @param float $amount
	 *
	 * @throws Mage_Core_Exception
	 */
	public function refund(Varien_Object $payment, $amount)
	{

		$api = new Api(
			Mage::getStoreConfig('payment/zen/terminal_api_key'),
			$this->getEnvironment()
		);

		$order = $payment->getOrder();

		$refund = $api->createRefund(
			$amount,
			$payment->getParentTransactionId(),
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId()
		);

		if ($refund['success']) {

			$order = $payment->getOrder();

			$comment = Mage::helper('core')->__('The refund has been requested. Amount:') . ' ' . $amount;

			$order->addStatusHistoryComment($comment)->save();

			Mage::throwException($comment);
		}

		Mage::throwException(Mage::helper('core')->__('The refund action is not available at this moment. Try again later.'));
	}

	/**
	 * @return string
	 */
	public function getEnvironment()
	{
		return Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @return Mage_Sales_Model_Order
	 *
	 */
	public function getOrder()
	{
		if (!$this->_order) {
			$this->_order = $this->getInfoInstance()->getOrder();
		}

		return $this->_order;
	}

	/**
	 * @return string
	 */
	public function getOrderPlaceRedirectUrl()
	{
		return Mage::getUrl('zen/processing/redirect');
	}

	/**
	 * @param string $currencyCode
	 *
	 * @return bool
	 */
	public function canUseForCurrency($currencyCode)
	{

		$currency = Mage::getStoreConfig('payment/zen/currency');

		if (!is_string($currency)) {
			return false;
		}

		return Util::canUseForCurrency(
				strtolower($currencyCode),
				explode(
					',',
					strtolower(
						str_replace(' ', '', $currency)
					)
				)
			)
			&& Mage::getStoreConfig('payment/zen/terminal_uuid')
			&& Mage::getStoreConfig('payment/zen/paywall_secret')
			&& Mage::getStoreConfig('payment/zen/terminal_api_key')
			&& Mage::getStoreConfig('payment/zen/ipn_secret');
	}

	/**
	 * @return array
	 */
	public function getPaywallData()
	{

		$order = $this->getOrder();
		$billingAddress = $order->getBillingAddress();
		$total = Util::convertAmount($this->getOrder()->getGrandTotal());

		return Paywall::prepareOrderData(
			$total,
			$order->getOrderCurrencyCode(),
			self::getOrderId(),
			$billingAddress->getFirstname(),
			$billingAddress->getLastname(),
			$billingAddress->getEmail(),
			Mage::getUrl('checkout/onepage/success'),
			Mage::getUrl('checkout/onepage/failure'),
			Mage::getUrl('checkout/onepage/success'),
			Mage::getUrl('zen/notification/process'),
			$this->getCart($order, $total),
			Mage::getStoreConfig('payment/zen/terminal_uuid'),
			Mage::getStoreConfig('payment/zen/paywall_secret'),
			(string) Mage::getConfig()->getNode('global/resources/zen_setup/setup/module'),
			(string) Mage::getConfig()->getNode('modules/zen/version'),
			'Magento',
			Mage::getVersion()
		);
	}

	/**
	 * @return string
	 */
	public function getOrderId()
	{
		return $this->getOrder()->getRealOrderId();
	}

	/**
	 * @param Mage_Sales_Model_Order $order
	 * @param int $orderTotal
	 *
	 * @return array
	 */
	private function getCart($order, $orderTotal)
	{

		$cartData = new CartData();

		$cartData->setAmount($orderTotal);

		foreach ($order->getAllItems() as $item) {
			$itemCost = Util::convertAmount($item->getPriceInclTax());
			$quantity = (float)$item->getQtyOrdered();

			$cartData->addItem(
				$item->getName(),
				$itemCost,
				$quantity,
                Util::convertAmount($itemCost * $quantity)
			);
		}

		$shippingDescription = $order->getShippingDescription();

		if (!$order->getIsVirtual() && $shippingDescription) {
			$shippingCost = Util::convertAmount($order->getShippingInclTax());

			$cartData->addItem(
				$shippingDescription,
				$shippingCost,
				1,
				$shippingCost
			);
		}

		$discount = Util::convertAmount($order->getBaseDiscountAmount());

		if ($discount != 0) {
			$cartData->addItem(
				__('Discount'),
				$discount,
				1,
				$discount
			);
		}

		return $cartData->getCartData();
	}

	/**
	 * @return string
	 */
	public function getApiKey()
	{
		return Mage::getStoreConfig('payment/zen/terminal_api_key');
	}
}
